require "numru/ggraph"
require 'getoptlong'
require "./gp_dcpam_const_v1.0"
require "./gp_dcpam_methods_v1.0"
include NumRu


parser = GetoptLong.new

parser.set_options(
  ['--yaxis',                    GetoptLong::REQUIRED_ARGUMENT],
  ['--cnttype',                  GetoptLong::REQUIRED_ARGUMENT],
  ['--vname-in-file',            GetoptLong::REQUIRED_ARGUMENT],
  ['--meantype',                 GetoptLong::REQUIRED_ARGUMENT],
)

begin
  parser.each_option do |name, arg|
    eval "$OPT_#{name.sub(/^--/, '').gsub(/-/, '_')} = '#{arg}'"
#    print name, ":", arg, "\n"
  end
rescue
  exit(1)
end

DaysInAYear = DaysOfMonth.sum

if ARGV.size == 4 then
  path     = ARGV[1]
  vname    = ARGV[2]
  path_ps  = ARGV[3]
else
  print "Unexpected number of arguments: ", ARGV.size, "\n"
  exit
end

yaxis         = ($OPT_yaxis||"lin").to_s
cnttype       = ($OPT_cnttype||"lin").to_s
meantype      = ($OPT_meantype||"year").to_s
vname_in_file = ($OPT_vname_in_file||vname).to_s

plev = PLev
if plev[0] < plev[1] then
  plev = plev.reverse
end



case yaxis
when "lin" then
  itr = 1
when "log" then
  itr = 2
else
  print "Unexpected type of y axis: ", yaxis, "\n"
  exit
end
case cnttype
when "lin" then
  flag_cntlog = false
when "log" then
  flag_cntlog = true
else
  print "Unexpected type of contour: ", cnttype, "\n"
  exit
end


ncfn = path
url = ncfn + "@" + vname_in_file
gphys = GPhys::IO.open_gturl( url )

ncfn = path_ps
url = ncfn + "@" + "Ps"
gp_ps = GPhys::IO.open_gturl( url )


na_lon     = gphys.coord('lon').val
na_lat     = gphys.coord('lat').val
na_sig     = gphys.coord('sig').val
na_time_in = gphys.coord('time').val

gp_sig = gphys.axis('sig').to_gphys

imax = na_lon.size
jmax = na_lat.size
kmax = na_sig.size
tmax = na_time_in.size


case meantype
when "year" then
  a_meanwidth = [ DaysInAYear * DataNumOfDay ]
when "month" then
  a_meanwidth = Array.new( DaysOfMonth.size )
  DaysOfMonth.each_with_index do |nDays, index|
    a_meanwidth[index] = nDays * DataNumOfDay
  end
end
nMon = a_meanwidth.length

imax = gphys.coord('lon').val.size
jmax = gphys.coord('lat').val.size
kmax = plev.size
tmax = nMon
na_AveArray = NArrayMiss.float(jmax,kmax,tmax)


ts0 = - DaysInAYear * DataNumOfDay
te0 = -1

te  = ts0-1
for iMon in 0..(nMon-1)

  nAve = a_meanwidth[iMon]

  ts = te  + 1
  te = ts  + nAve-1
  te = [te0,te].min

  print "  iMon: ", iMon, ", ts: ", ts, ", te: ", te, "\n"

  tmax_sl = 1000
  for iLoop in 0..((te-ts-1)/tmax_sl)
    ts_sl = ts + tmax_sl * iLoop
    te_sl = ts_sl + tmax_sl - 1
    te_sl = [te_sl,te].min
#  print iLoop, ", ", ts_sl, ", ", te_sl, "\n"

    # pickup part of GPhys object (array)
    gp_part    = gphys[true,true,true,ts_sl..te_sl].copy
    gp_ps_part = gp_ps[true,true,ts_sl..te_sl].copy

    gp_part_onplev = interpolate_on_p( plev, gp_ps_part, gp_part )

    case vname
    when "StatStab" then
      gp_part_onplev =
        calc_statstab( gp_part_onplev, Grav, GasRDry, CpDry, 'pressure' )
    end

    if iLoop == 0 then
      gpout_tmp = gp_part_onplev[true,true,true,0].copy
      gpout_tmp[true,true,true] = 0.0
      gp_mask = gpout_tmp.copy
      gp_mask[true,true,true] = 0
    end

    gpout_tmp = gpout_tmp + gp_part_onplev.sum('time')
    gp_mask[true,true,true] = gp_mask[true,true,true] +
                              gp_part_onplev.val.get_mask.to_f.sum(3)

  end
  gp_mask[ ( gp_mask.val.eq 0 ) ] = 1
  gpout_tmp = gpout_tmp / gp_mask

  gp_mask[true,true,true] = gpout_tmp.val.get_mask.to_f
  gp_mask = gp_mask.sum('lon')
  gp_mask[ ( gp_mask.val.eq 0 ) ] = 1
  gpout_tmp = gpout_tmp.sum('lon') / gp_mask

  na_AveArray[true,true,iMon] = gpout_tmp.val[true,true]

end

ax_lat      = gphys.axis('lat')
ax_plev     = gpout_tmp.axis('pressure')
na_time_out = NArray.sfloat(nMon).indgen(1,1)
va_time     = VArray.new( na_time_out, 
                          {"long_name"=>"time", "units"=>"months"},
                          "time" )
ax_time     = Axis.new.set_pos(va_time)
va_data     = VArray.new( na_AveArray, 
                          {"long_name"=>gphys.long_name,
                           "units"=>gphys.units.to_s},
                          gphys.name )
gpout = GPhys.new( Grid.new(ax_lat,ax_plev,ax_time), va_data )


case vname
when "AngMom" then
  for iMon in 0..(nMon-1)
    gpout[true,true,iMon] =
      calc_absangmom(
        RPlanet, Omega, gpout[true,true,iMon], "lat", "pressure"
      )
  end
  gpout = gpout * 1.0e-9
  gpout.units = "1e9 " + gpout.units.to_s
when "MSF" then
  for iMon in 0..(nMon-1)
    gpout[true,true,iMon] =
      calc_msf(
        Grav, RPlanet, gpout[true,true,iMon], 'lat', 'pressure'
      )
  end
  gpout = gpout * 1.0e-10
  gpout.units = "1e10 " + gpout.units.to_s
when "PotTemp" then
  for iMon in 0..(nMon-1)
    gpout[true,true,iMon] =
      calc_pottemp( gpout[true,true,iMon], GasRDry/CpDry, Press00, 'pressure' )
  end
end



iws = (ARGV[0] || (puts ' WORKSTATION ID (I)  ? ;'; DCL::sgpwsn; gets)).to_i
DCL.gropn(iws)

#DCL.sldiv('y',1,4)
DCL.sgpset('isub', 96)     # control character of subscription: '_' --> '`'
DCL.sgpset('lfull',true)
DCL.uzfact(0.6)            # font size
DCL.sgpset('lclip', true)  # clip graph
DCL.glpset('lmiss',true)   # missing value

#DCL.sgscmn(4)  # blue-cyan-white-yellow-red
#DCL.sgscmn(5)  # gray-scale
#DCL.sgscmn(14) # blue-white-red

#levels = [110,120,130,140,150,160,170,180,190,200,210,220,230,240]
#patterns = [10999,15999,20999,25999,30999,35999,40999,50999,60999,65999,70999,75999,80999,90999,95999]

if flag_cntlog then
  cntmin = nil
  cntmax = nil
else
  cntmin = gpout.min
  cntmax = gpout.max
end
for iMon in 0..(nMon-1)

  GGraph.set_fig 'itr'=> itr, 'viewport'=>[0.1,0.9,0.2,0.6], 'window'=>[-90,90,plev[0],plev[-1]]
  #GGraph.set_map "coast_world"=>true, 'grid'=>false
  GGraph.set_axes('xlabelint'=>30)
  #GGraph.set_axes('ylabelint'=>30)

  GGraph.tone    gpout[true,true,iMon], true , 'exchange'=>false,
                 'title'=>vname, 
                 'map_axes'=>true, 'annotate'=>false,
                 'log'=>flag_cntlog, 
                 'nlev'=>20, 'keep'=>true, 
                 'min'=>cntmin, 'max'=>cntmax #, 'lev'=>levels, 'pat'=>patterns
  GGraph.contour gpout[true,true,iMon], false, 'exchange'=>false,
                 'log'=>flag_cntlog, 
                 'nlev'=>20, 'keep'=>true, 
                 'min'=>cntmin, 'max'=>cntmax #, 'lev'=>levels
  GGraph.color_bar

  if nMon > 1 then
    title = (iMon+1).to_s + "/" + nMon.to_s
    DCL::uxmttl('T', ' '  , 1.0)
    DCL::uxmttl('T', title, -1.0)
  end
end

DCL.grcls
